@@ -2,7 +2,7 @@
  * ssh_cmd.c -- sftp support via libssh.
  *
  * Yet Another FTP Client
- * Copyright (C) 2012, Sebastian Ramacher <sebastian+dev@ramacher.at>
+ * Copyright (C) 2012,2015 Sebastian Ramacher <sebastian+dev@ramacher.at>
  * Copyright (C) 1998-2001, Martin Hedenfalk <mhe@stacken.kth.se>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,12 +17,6 @@
 #include "gvars.h"
 #include "libmhe/args.h"
 #include "rfile.h"
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
 
 #if defined(BUFSIZ) && BUFSIZ >= 131072
 #define SSH_BUFSIZ BUFSIZ
@@ -33,23 +27,27 @@
 /* from libssh examples */
 static int verify_knownhost(ssh_session session)
 {
-  unsigned char *hash = NULL;
+  const int state = ssh_is_server_known(session);
 
-  int state = ssh_is_server_known(session);
-  int hlen = ssh_get_pubkey_hash(session, &hash);
-  if (hlen < 0)
+  ssh_key pubkey;
+  int rc = ssh_get_publickey(session, &pubkey);
+  if (rc < 0)
+    return -1;
+
+  unsigned char *hash = NULL;
+  size_t hlen = 0;
+  rc = ssh_get_publickey_hash(pubkey, SSH_PUBLICKEY_HASH_SHA1, &hash, &hlen);
+  ssh_key_free(pubkey);
+  if (rc < 0)
     return -1;
 
   switch (state)
   {
-    case SSH_SERVER_KNOWN_OK:
-      break; /* ok */
-
     case SSH_SERVER_KNOWN_CHANGED:
       fprintf(stderr, _("Host key for server changed. It is now:\n"));
       ssh_print_hexa(_("Public key hash"), hash, hlen);
       fprintf(stderr, _("For security reasons, connection will be stopped.\n"));
-      free(hash);
+      ssh_clean_pubkey_hash(&hash);
       return -1;
 
     case SSH_SERVER_FOUND_OTHER:
@@ -57,7 +55,7 @@ static int verify_knownhost(ssh_session session)
         "another type of key exists.\n"));
       fprintf(stderr, _("An attacker might change the default server key to "
         "confuse your client into thinking the key does not exist\n"));
-      free(hash);
+      ssh_clean_pubkey_hash(&hash);
       return -1;
 
     case SSH_SERVER_FILE_NOT_FOUND:
@@ -77,26 +75,25 @@ static int verify_knownhost(ssh_session session)
       char buf[64] = { '\0' };
       if (fgets(buf, sizeof(buf), stdin) == NULL)
       {
-        free(hash);
+        ssh_clean_pubkey_hash(&hash);
         return -1;
       }
 
       /* replace \n from fgets with \0 */
       const size_t len = strlen(buf);
-      if (len > 0 && buf[len - 1] == '\n') {
+      if (len > 0 && buf[len - 1] == '\n')
         buf[len - 1] = '\0';
-      }
 
       if (strcasecmp(buf, _("yes")) != 0 && strcasecmp(buf, "yes") != 0)
       {
-        free(hash);
+        ssh_clean_pubkey_hash(&hash);
         return -1;
       }
 
       if (ssh_write_knownhost(session) < 0)
       {
         fprintf(stderr, _("Error %s\n"), strerror(errno));
-        free(hash);
+        ssh_clean_pubkey_hash(&hash);
         return -1;
       }
       break;
@@ -104,11 +101,11 @@ static int verify_knownhost(ssh_session session)
 
     case SSH_SERVER_ERROR:
       fprintf(stderr, _("Error %s\n"), ssh_get_error(session));
-      free(hash);
+      ssh_clean_pubkey_hash(&hash);
       return -1;
   }
 
-  free(hash);
+  ssh_clean_pubkey_hash(&hash);
   return 0;
 }
 
@@ -193,7 +190,8 @@ static int authenticate_kbdint(ssh_session session)
 static int test_several_auth_methods(ssh_session session, url_t* urlp)
 {
   int rc = ssh_userauth_none(session, NULL);
-  /* if (rc != SSH_AUTH_SUCCESS) {
+  /* if (rc != SSH_AUTH_SUCCESS)
+     {
       return rc;
   } */
 
@@ -235,12 +233,14 @@ int ssh_open_url(url_t* urlp)
   }
 
   /* If we have ssh options from yafcrc, load them */
-  if (gvSSHOptions) {
+  if (gvSSHOptions)
+  {
     args_t *args = args_create();
     args_init(args, 0, NULL);
     args_push_back(args, gvSSHOptions);
     int argc = 0;
-    if (ssh_options_getopt(ftp->session, &argc, args->argv) != SSH_OK) {
+    if (ssh_options_getopt(ftp->session, &argc, args->argv) != SSH_OK)
+    {
       ftp_err(_("Failed to load SSH options from yafcrc config (ssh_options = '%s')\n"), gvSSHOptions);
       ssh_free(ftp->session);
       ftp->session = NULL;
@@ -301,7 +301,8 @@ int ssh_open_url(url_t* urlp)
   }
 
   ftp->ssh_version = ssh_get_version(ftp->session);
-  if (!ftp->ssh_version) {
+  if (!ftp->ssh_version)
+  {
     ftp_err(_("Couldn't initialise connection to server\n"));
     return -1;
   }
@@ -378,7 +379,8 @@ int ssh_chdir(const char *path)
 
   stripslash(p);
   isdir = (ftp_cache_get_directory(p) != 0);
-  if(!isdir) {
+  if(!isdir)
+  {
     rfile *rf = ftp_cache_get_file(p);
     isdir = (rf && risdir(rf));
   }
@@ -391,7 +393,8 @@ int ssh_chdir(const char *path)
       free(p);
       return -1;
     }
-    if (!S_ISDIR(attrib->permissions)) {
+    if (!S_ISDIR(attrib->permissions))
+    {
       ftp_err(_("%s: not a directory\n"), p);
       sftp_attributes_free(attrib);
       free(p);
@@ -491,11 +494,17 @@ int ssh_help(const char *arg)
   return 0;
 }
 
-uint64_t ssh_filesize(const char *path)
+uint64_t ssh_filesize(const char* path)
 {
-  sftp_attributes attrib = sftp_stat(ftp->sftp_session, path);
+  char* p = ftp_path_absolute(path);
+  sftp_attributes attrib = sftp_stat(ftp->sftp_session, p);
   if (!attrib)
+  {
+    ftp_trace("Couldn't stat file '%s': %s\n", p, ssh_get_error(ftp->session));
+    free(p);
     return 0;
+  }
+  free(p);
 
   uint64_t res = attrib->size;
   sftp_attributes_free(attrib);
@@ -594,11 +603,18 @@ int ssh_rename(const char *oldname, const char *newname)
   return 0;
 }
 
-time_t ssh_filetime(const char *filename)
+time_t ssh_filetime(const char* filename)
 {
-  sftp_attributes attrib = sftp_stat(ftp->sftp_session, filename);
+  char* path = ftp_path_absolute(filename);
+  sftp_attributes attrib = sftp_stat(ftp->sftp_session, path);
   if (!attrib)
+  {
+    ftp_trace("Couldn't stat file '%s': %s\n", path,
+              ssh_get_error(ftp->session));
+    free(path);
     return -1;
+  }
+  free(path);
 
   time_t res = attrib->mtime; /* mtime 64? */
   sftp_attributes_free(attrib);
@@ -996,7 +1012,8 @@ int ssh_send(const char *path, FILE *fp, putmode_t how,
 
   ftp->ti.transfer_is_put = true;
 
-  if(how == putUnique) {
+  if (how == putUnique)
+  {
     ftp_err(_("Unique put with SSH not implemented yet\n"));
     return -1;
   }
@@ -1005,7 +1022,8 @@ int ssh_send(const char *path, FILE *fp, putmode_t how,
   stripslash(p);
   ftp_cache_flush_mark_for(p);
 
-  if(how == putAppend) {
+  if (how == putAppend)
+  {
     ftp_set_tmp_verbosity(vbNone);
     offset = ftp_filesize(p);
   }
