@@ -16,6 +16,79 @@
 #include "shortpath.h"
 #include "gvars.h"
 
+typedef struct string_buffer_s
+{
+  char* start;
+  size_t current;
+  size_t size;
+} string_buffer_t;
+
+static string_buffer_t* sb_new(void)
+{
+  string_buffer_t* sb = malloc(sizeof(string_buffer_t));
+  if (!sb)
+    return NULL;
+
+  const size_t new_size = 128;
+  sb->start = calloc(1, new_size);
+  if (!sb->start)
+  {
+    free(sb);
+    return NULL;
+  }
+
+  sb->current = 0;
+  sb->size = new_size;
+
+  return sb;
+}
+
+static bool sb_grow(string_buffer_t* sb, const size_t size)
+{
+  const size_t new_size = sb->size + ((size / 128) + 1) * 128;
+  char* tmp = realloc(sb->start, new_size);
+  if (!tmp)
+    return false;
+
+  memset(tmp + sb->current, '\0', new_size - sb->current);
+
+  sb->start = tmp;
+  sb->size = new_size;
+
+  return true;
+}
+
+static bool sb_append_c(string_buffer_t* sb, char c)
+{
+  if (sb->current + 1 == sb->size)
+    if (!sb_grow(sb, 1))
+      return false;
+
+  sb->start[sb->current] = c;
+  ++sb->current;
+
+  return true;
+}
+
+static bool sb_append_s(string_buffer_t* sb, const char* s)
+{
+  const size_t len = strlen(s);
+  if (sb->current + len + 1 >= sb->size)
+    if (!sb_grow(sb, len))
+      return false;
+
+  strcat(sb->start + sb->current, s);
+  sb->current += len;
+
+  return true;
+}
+
+static void sb_free(string_buffer_t* sb)
+{
+  free(sb->start);
+  free(sb);
+}
+
 static int connection_number(void)
 {
   int i = 1;
@@ -52,7 +125,8 @@ static int connection_number(void)
  *  example: %32w
  */
 
-static bool read_uint(const char** str, unsigned int* number) {
+static bool read_uint(const char** str, unsigned int* number)
+{
   if (!str || !*str || !number) {
     return false;
   }
@@ -80,8 +154,12 @@ char* expand_prompt(const char* fmt)
   if (!fmt)
     return NULL;
 
-  char* prompt = xmalloc(strlen(fmt) + 1);
-  char* cp = prompt;
+  string_buffer_t* sb = sb_new();
+  if (!sb)
+  {
+    fprintf(stderr, _("Failed to allocate memory.\n"));
+    return NULL;
+  }
 
   while (fmt && *fmt)
   {
@@ -90,7 +168,7 @@ char* expand_prompt(const char* fmt)
       fmt++;
 
       char* ins = NULL;
-      bool freeins = false;
+      const char* cins = NULL;
       unsigned int maxlen = UINT_MAX;
 
       read_uint(&fmt, &maxlen);
@@ -102,10 +180,9 @@ char* expand_prompt(const char* fmt)
           if (asprintf(&ins, "%u", list_numitem(gvFtpList)) == -1)
           {
             fprintf(stderr, _("Failed to allocate memory.\n"));
-            free(prompt);
+            sb_free(sb);
             return NULL;
           }
-          freeins = true;
           break;
         }
 
@@ -114,10 +191,9 @@ char* expand_prompt(const char* fmt)
           if (asprintf(&ins, "%u", connection_number()) == -1)
           {
             fprintf(stderr, _("Failed to allocate memory.\n"));
-            free(prompt);
+            sb_free(sb);
             return NULL;
           }
-          freeins = true;
           break;
         }
 
@@ -125,50 +201,43 @@ char* expand_prompt(const char* fmt)
         {
           char* user = ftp_connected_user();
           if (user)
-          {
             ins = user;
-            freeins = true;
-          }
           else
-            ins = "?";
+            cins = "?";
           break;
         }
 
         case 'h': /* remote hostname (as passed to cmd_open()) */
         {
-          ins = ftp_connected() ? ftp->url->hostname : "?";
+          cins = ftp_connected() ? ftp->url->hostname : "?";
           break;
         }
 
         case 'H': /* %h up to first '.' */
         {
           if (!ftp_connected()) {
-            ins = "?";
+            cins = "?";
             break;
           }
 
           char* e = strchr(ftp->url->hostname, '.');
           if (e)
-          {
             ins = xstrndup(ftp->url->hostname, e - ftp->url->hostname);
-            freeins = true;
-          }
           else
-            ins = ftp->url->hostname;
+            cins = ftp->url->hostname;
           break;
         }
 
         case 'm': /* remote machine name (as returned from gethostbynmame) */
         {
           ins = xstrdup(ftp_connected() ? host_getoname(ftp->host) : "?");
-          freeins = true;
           break;
         }
 
         case 'M': /* %m up to first '.' */
         {
           if (!ftp_connected()) {
-            ins = "?";
+            cins = "?";
             break;
           }
 
@@ -178,33 +247,28 @@ char* expand_prompt(const char* fmt)
                      e - host_getoname(ftp->host));
           else
             ins = xstrdup(host_getoname(ftp->host));
-          freeins = true;
           break;
         }
 
         case 'n': /* remote ip number */
         {
           ins = xstrdup(ftp_connected() ? host_getoname(ftp->host) : "?");
-          freeins = true;
           break;
         }
 
         case 'w': /* current remote directory */
         {
           if (!ftp_loggedin())
-            ins = "?";
+            cins = "?";
           else
-          {
             ins = shortpath(ftp->curdir, maxlen, 0);
-            freeins = true;
-          }
           break;
         }
 
         case 'W': /* basename(%w) */
         {
           if (!ftp_loggedin())
-            ins = "?";
+            cins = "?";
           else
             ins = (char *)base_name_ptr(ftp->curdir);
           break;
@@ -213,12 +277,9 @@ char* expand_prompt(const char* fmt)
         case '~': /* %w but homedir replaced with '~' */
         {
           if (!ftp_loggedin())
-            ins = "?";
+            cins = "?";
           else
-          {
             ins = shortpath(ftp->curdir, maxlen, ftp->homedir);
-            freeins = true;
-          }
           break;
         }
 
@@ -228,7 +289,6 @@ char* expand_prompt(const char* fmt)
           if (tmp)
           {
             ins = shortpath(tmp, maxlen, 0);
-            freeins = true;
             free(tmp);
           }
           break;
@@ -240,7 +300,6 @@ char* expand_prompt(const char* fmt)
           if (tmp)
           {
             ins = base_name_xptr(tmp);
-            freeins = true;
             free(tmp);
           }
           break;
@@ -248,56 +307,66 @@ char* expand_prompt(const char* fmt)
 
         case '%': /* percent sign */
         {
-          ins = "%";
+          cins = "%";
           break;
         }
 
         case '#': /* local user == root ? '#' : '$' */
         {
-          ins = getuid() == 0 ? "#" : "$";
+          cins = getuid() == 0 ? "#" : "$";
           break;
         }
 
 #ifdef HAVE_LIBREADLINE
         case '{': /* begin non-printable character string */
         {
-          ins = "\001"; /* RL_PROMPT_START_IGNORE */
+          cins = "\001"; /* RL_PROMPT_START_IGNORE */
           break;
         }
 
         case '}': /* end non-printable character string */
         {
-          ins = "\002"; /* RL_PROMPT_END_IGNORE */
+          cins = "\002"; /* RL_PROMPT_END_IGNORE */
           break;
         }
 #endif
 
         case 'e': /* escape (0x1B) */
         {
-          ins = "\x1B";
+          cins = "\x1B";
           break;
         }
       }
 
-      if (ins)
+      if (ins || cins)
       {
-        const size_t len = strlen(prompt) + strlen(ins) + strlen(fmt+1) + 1;
-        char* tmp = xmalloc(len);
-        strlcpy(tmp, prompt, len);
-        strlcat(tmp, ins, len);
-        cp = tmp + strlen(prompt) + strlen(ins);
-        free(prompt);
-        prompt = tmp;
-        if (freeins)
-          free(ins);
+        const bool ret = ins ? sb_append_s(sb, ins) : sb_append_s(sb, cins);
+        free(ins);
+
+        if (!ret)
+        {
+          fprintf(stderr, _("Failed to allocate memory.\n"));
+          sb_free(sb);
+          return NULL;
+        }
       }
     }
     else
-      *cp++ = *fmt;
+    {
+      if (!sb_append_c(sb, *fmt))
+      {
+        fprintf(stderr, _("Failed to allocate memory.\n"));
+        sb_free(sb);
+        return NULL;
+      }
+    }
 
     fmt++;
   }
 
+  char* prompt = xstrdup(sb->start);
+  sb_free(sb);
+
   unquote_escapes(prompt);
   return prompt;
 }
